#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# All rights reserved.
#
# This code is licensed under the MIT License.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files(the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and / or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions :
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import requests
import json
import re
import os
import math
from datetime import datetime
from datetime import timedelta
from .common import (
    AlarmScene, AreaStatus, AreaBank, ZoneStatus, ZoneBank, ALARM_SCENES,
    AREA_STATES, ZONE_STATES, PanelFunction)
from .config import UltraSyncConfig
from urllib.parse import unquote
from .logger import logger


class UltraSync(UltraSyncConfig):
    """
    A wrapper to the UltraSync Alarm Panel
    """

    panel_encoding = 'utf-8'

    # Requests timeout needs to be high as the panel is not always the most
    # responsive
    timeout = (8, 10)

    def __init__(self, *args, **kwargs):
        """
        prepares our UltraSync object
        """

        super(UltraSync, self).__init__(*args, **kwargs)

        # Create our session object
        self.session = requests.Session()

        # Our session id is retrieved after a successful login
        self.session_id = None

        # User Status codes
        self.__is_master = None
        self.__is_installer = None

        # Taken straight out of status.js
        self.__area_state_byte = [
            6, 4, 0, 16, 20, 18, 22, 8, 10, 12, 64, 66, 68, 70, 72, 14, 56]

        # Our zones get populated after we connect
        self.zones = {}
        self.__zbank = None
        self.__zsequence = None

        # Virtual bank for tracking individual sensor
        self.__zvbank = {}

        # Our areas get populated after we connect
        self.areas = {}

        # Track the time our information was polled from our panel
        self.__updated = None

        # Track the Panel URL path
        self.__panel_url_path = None

    def login(self):
        """
        Performs login to UltraSync (which then redirects to area.htm)

        """
        # Our initialiation
        self.session_id = None
        self.__updated = None

        payload = {
            'lgname': self.user,
            'lgpin': self.pin,
        }

        logger.info('Authenticating to {}'.format(self.host))
        response = self.__get(
            '/login.cgi', payload=payload, is_json=False, auth_on_fail=False)
        if not response:
            logger.error('Failed to authenticate to {}'.format(self.host))
            return False

        #
        # Get our Session Identifier
        #
        # It looks like this in the login.cgi response:
        #  function getSession(){return "A2D6C62695D705D8";}
        match = re.search(
            r'function getSession\(\)[^"]+"(?P<session>[^"]+)".*',
            response, re.M)
        if not match:
            # No match and/or bad login
            logger.error('Failed to authenticate to {}'.format(self.host))
            return False

        # Store our session
        self.session_id = match.group('session')

        #
        # Get our Panel URL Path
        #
        # It is parsed from a reference like
        #   script src="/v_ZW_03.02-C/status.js"
        #                     ^
        #                     |
        match = re.search(
            r'script src="(?P<path>/?[^/]+).*', response, re.M)
        if not match:
            # No match and/or bad login
            return False

        # Store our path
        self.__panel_url_path = match.group('path')

        if not self._areas(response=response) or not self._zones():
            # No match and/or bad login
            logger.error('Failed to authenticate to {}'.format(self.host))
            return False

        # Update our time reference
        self.__updated = datetime.now()

        # We're good
        return True if self.session_id else False

    def logout(self):
        """
        Performs login to UltraSync

        """
        if not self.session_id:
            # We're done
            return True

        logger.info('Graceful log off from {}'.format(self.host))
        # Reset our variables reguardless if we're successfully able to log
        # out or not
        self.session_id = None

        # Perform a logout
        response = self.__get('/logout.cgi', is_json=False)
        if not response:
            logger.error('Failed to authenticate to {}'.format(self.host))
            return False

        return True

    def debug_dump(self, path=None, mode=0o755):
        """
        Useful for checking for differences in alarm readings over time.

        """
        if path is None:
            path = datetime.now().strftime('%Y%m%d%H%M%S.ultrasync-dump')

        # Begin capturing data
        if not self.session_id and not self.login():
            return False

        logger.info('Performing a debug dump to: {}'.format(path))
        os.mkdir(path, mode=mode)

        urls = {
            # status.json - Bank 0
            'status.0.json': {
                'path': '/user/status.json',
                'payload': {
                    'sess': self.session_id,
                    'arsel': 0,
                },
            },
            # status.json - Bank 1
            'status.1.json': {
                'path': '/user/status.json',
                'payload': {
                    'sess': self.session_id,
                    'arsel': 1,
                },
            },
            # Area URL
            'area.htm': {
                'path': '/user/area.htm',
            },

            # Zones URL
            'zones.htm': {
                'path': '/user/zones.htm',
            },

            # Room URL
            'rooms.htm': {
                'path': '/user/rooms.htm',
            },

            # Zones URL
            'addrem.json': {
                'path': '/user/zones.htm',
            },

            # Used to acquire sequence
            'seq.json': {
                'path': '/user/seq.json',
            },

            # Config Main Screen
            'config2.htm': {
                'path': '/muser/config2.htm',
            },

            # Additional useful queries
            'master.js': {
                'path': '{}/master.js'.format(self.__panel_url_path),
                'method': 'GET'
            },
            'zwave.js': {
                'path': '{}/zwave.js'.format(self.__panel_url_path),
                'method': 'GET'
            },
            'status.js': {
                'path': '{}/status.js'.format(self.__panel_url_path),
                'method': 'GET'
            },
            'connect.xml': {
                'path': '/protect/randmenu.xml',
                'payload': {
                    'sess': self.session_id,
                    'item': 1,
                    'minc': 'protect/connect.xml'
                },
            },
        }

        # Scene Captures
        urls.update({'scenes.{}.xml'.format(no + 1): {
            'path': '/protect/randmenu.xml',
            'payload': {
                'sess': self.session_id,
                'item': no + 1,
                'minc': 'protect/scenes.xml'
            }} for no in range(0, 16)})

        # Sensor Captures
        urls.update({'sensor.{}.xml'.format(no + 1): {
            'path': '/protect/randmenu.xml',
            'payload': {
                'sess': self.session_id,
                'item': no + 1,
                'minc': 'protect/sensor.xml'
            }} for no in range(0, 16)})

        # Area Captures
        urls.update({'area.{}.xml'.format(no + 1): {
            'path': '/protect/randmenu.xml',
            'payload': {
                'sess': self.session_id,
                'item': no + 1,
                'minc': 'protect/area.xml'
            }} for no in range(0, 4)})

        for to_file, kwargs in urls.items():
            response = self.__get(is_json=False, **kwargs)
            if not response:
                continue

            with open(os.path.join(path, to_file), 'w',
                      encoding=self.panel_encoding) as fp:
                # Write our content to disk
                _bytes = fp.write(response)
                logger.info('Wrote {} bytes to {}'.format(_bytes, to_file))

    def set(self, area=1, state=AlarmScene.DISARMED):
        """
        Sets Alarm Scene

        """
        if not self.session_id and not self.login():
            return False

        if state not in ALARM_SCENES:
            return False

        mask = 1 << (area - 1) % 8
        start = math.floor((area - 1) / 8)

        payload = {
            'sess': self.session_id,
            'start': int(start),
            'mask': mask,
        }

        if state == AlarmScene.STAY:
            payload.update({
                'fnum': PanelFunction.AREA_STAY,
            })

        elif state == AlarmScene.AWAY:
            payload.update({
                'fnum': PanelFunction.AREA_AWAY,
            })

        else:   # AlarmScene.DISARMED
            payload.update({
                'fnum': PanelFunction.AREA_DISARM,
            })

        response = self.__get('/user/keyfunction.cgi', payload=payload)
        if not response:
            return False

        return response

    def update(self, ref=None, max_age_sec=1):
        """
        Updates classmeta information
        """

        if not self.session_id and not self.login():
            return False

        if ref is None:
            ref = datetime.now()

        if (self.__updated + timedelta(seconds=max_age_sec)) <= ref:
            # Perform a sequence check; these trigger subsequent
            # checks if required
            if not self._sequence():
                return False

        else:
            # No update required
            return True

        # Update our time reference
        self.__updated = datetime.now()
        return True

    def details(self):
        """
        Arranges the areas and zones into an easy to manage dictionary
        """
        if not self.update():
            return {}

        # Build our response object
        response = {
            'user': {
                'is_installer': self.__is_installer,
                'is_master': self.__is_master,
            },
            'zones': [z for z in self.zones.values()],
            'areas': [a for a in self.areas.values()],
            'date': self.__updated.strftime('%Y-%m-%d %H:%M:%S'),
        }

        return response

    def _areas(self, response=None):
        """
        Arranges the areas into an easier to read dictionary; since the
        login.cgi redirects users to this page by default, we allow users to
        pass a response object.

        the area.htm is rather a heavy set page, if we've already populated our
        area_names object then we can use a ligher query and just use the
        seq.json file to populate the same data.

        """
        logger.info('Retrieving initial Area information.')
        if not response:
            # No response object was specified; we need to query the area.htm
            # page.
            if not self.session_id and not self.login():
                return False

            # Perform our Query
            response = self.__get('/user/area.htm', is_json=False)
            if not response:
                return False

        #
        # Get our Area Sequence
        #
        # It looks like this in the login.cgi response:
        #  var areaSequence = [149,0,0,0,0,0,0,0,0,0,0,0];
        match = re.search(
            r'var areaSequence\s*=\s*(?P<sequence>[^]]+]).*', response, re.M)
        if not match:
            # No match and/or bad login
            return False
        sequence = json.loads(match.group('sequence'))

        #
        # Get our Area Status (Bank States)
        #
        # It looks like this in the login.cgi response:
        #  var areaStatus = ["0100000000...000000"];
        match = re.search(
            r'var areaStatus\s*=\s*(?P<states>[^]]+]).*', response, re.M)
        if not match:
            # No match and/or bad login
            return False
        bank_states = json.loads(match.group('states'))

        #
        # Get our Area Names
        #
        # It looks like this in the login.cgi response:
        #  var areaNames = ["","%21","%21","%21","%21","%21","%21","%21"];
        match = re.search(
            r'var areaNames\s*=\s*(?P<area_names>[^]]+]);.*', response, re.M)
        if not match:
            # No match and/or bad login
            return False

        # Store our Areas ('%21' == '!'; these are un-used areas)
        self.areas = \
            {x: {'name': unquote(y).strip()
                 if unquote(y).strip() else 'Area {}'.format(x + 1),
                 'bank': x,
                 'sequence': sequence[x],
                 'bank_state': bank_states[x]}
             for x, y in enumerate(json.loads(match.group('area_names')))
             if y != '%21'}

        return self.process_areas()

    def process_areas(self):
        """
        Updates our area information based on new configuration

        """
        # The following was reverse-engineered from status.js
        # from the function updateArea():
        for bank, area in self.areas.items():

            # some globals
            mask = 1 << bank % 8

            # prepare ourselves a virtual states for reference that can be
            # later indexed by name for readability.
            # Basically in status.js all references are by bank[idx:idx + 2];
            # This just translate mappings to that:
            #   0:2 - index 0
            #   2:4 - index 1
            #   4:6 - index 2
            #   ...
            #   78:80 - index 39
            vbank = [int(area['bank_state'][s:s + 2], 16) & mask
                     for s in range(0, 80, 2)]

            # Partially Armed State
            st_partial = bool(vbank[AreaBank.PARTIAL])

            # Armed State
            st_armed = bool(vbank[AreaBank.ARMED])

            # Exit Mode
            st_exit1 = bool(vbank[AreaBank.EXIT_MODE01])
            st_exit2 = bool(vbank[AreaBank.EXIT_MODE02])

            # Chime Set
            st_chime = bool(vbank[AreaBank.CHIME])

            # ?
            st_night = bool(vbank[AreaBank.NIGHT])

            # Priority, the lower, the higher it is; 6 being the lowest
            priority = 6

            # Now we'll attempt to detect our status
            status = None

            # Our initial index starting point
            bank_no = AreaBank.ARMED \
                if st_exit1 or st_exit2 else AreaBank.UNKWN_00

            while not status:

                if bank_no >= len(AREA_STATES):
                    # Set status
                    status = AreaStatus.READY
                    break

                # The old working way (based on status.js
                # bool(int(bank_state[self.__area_state_byte[bank_no]:
                #      self.__area_state_byte[bank_no] + 2], 16))
                if vbank[int(self.__area_state_byte[bank_no] / 2)]:
                    if st_partial:
                        status = AREA_STATES[bank_no]
                        if status in (AreaStatus.ARMED_STAY,
                                      AreaStatus.EXIT_DELAY_1,
                                      AreaStatus.EXIT_DELAY_2):
                            if st_night:
                                status += ' - Night'

                            elif vbank[AreaBank.UNKWN_07]:
                                status += ' - Instant'

                    if status == AreaStatus.EXIT_DELAY_1:
                        # Bump to EXIT_DELAY_2; we'll eventually hit
                        # the bottom of our while loop and move past that too
                        bank_no += 1

                elif AREA_STATES[bank_no] == AreaStatus.READY \
                        and not (st_armed or st_partial):
                    # Update
                    status = AreaStatus.NOT_READY \
                        if not vbank[AreaBank.UNKWN_01] \
                        else AreaStatus.NOT_READY_FORCEABLE

                # increment our index by one
                bank_no += 1

            if vbank[AreaBank.UNKWN_08] or vbank[AreaBank.UNKWN_09] or \
                    vbank[AreaBank.UNKWN_10] or vbank[AreaBank.UNKWN_11]:

                # Assign priority to 1
                priority = 1

            elif vbank[AreaBank.UNKWN_33] or vbank[AreaBank.UNKWN_34] or \
                    vbank[AreaBank.UNKWN_35] or vbank[AreaBank.UNKWN_36]:

                # Assign priority to 2
                priority = 2

            elif st_partial or vbank[AreaBank.UNKWN_32]:
                # Assign priority to 3
                priority = 3

            elif st_armed:
                # Assign priority to 4
                priority = 4

            elif not vbank[AreaBank.UNKWN_00]:
                # Assign priority to 5
                priority = 5

            area.update({
                'priority': priority,
                'states': {
                    'armed': st_armed,
                    'partial': st_partial,
                    'night': st_night,
                    'chime': st_chime,
                    'exit1': st_exit1,
                    'exit2': st_exit2,
                },
                'status': status,
            })

        return True

    def process_zones(self):
        """
        Updates our zone/sensor information based on new configuration

        """

        # The following was reverse-engineered from status.js
        # from the function updateArea():
        for bank, zone in self.zones.items():

            # some globals
            mask = 1 << bank % 8

            # Our initial offset
            idx = math.floor(bank / 8) * 2

            # Priority, the lower, the higher it is; 5 being the lowest
            priority = 5

            # prepare ourselves a virtual states for reference
            vbank = [int(self.__zbank[s][idx:idx + 2], 16) & mask
                     for s in range(0, 18)]

            # Update our virtual bank
            self.__zvbank[bank] = ''.join(
                [str(1 if b else 0) for b in vbank])

            # Track whether or not element is part of things
            can_bypass = not vbank[ZoneBank.BYPASS_DISABLED]

            if vbank[ZoneBank.UNKWN_05]:
                # red
                priority = 1

            elif vbank[ZoneBank.UNKWN_01] or vbank[ZoneBank.UNKWN_02] or \
                    vbank[ZoneBank.UNKWN_06] or vbank[ZoneBank.UNKWN_07] or \
                    vbank[ZoneBank.UNKWN_08] or vbank[ZoneBank.UNKWN_12]:
                # blue
                priority = 2

            elif vbank[ZoneBank.UNKWN_03] or vbank[ZoneBank.UNKWN_04]:
                # yellow
                priority = 3

            elif vbank[ZoneBank.UNKWN_00]:
                # grey
                priority = 4

            bank_no = 0

            status = None
            while not status:
                if vbank[bank_no]:
                    status = ZONE_STATES[bank_no]

                elif bank_no == 0:
                    status = ZoneStatus.READY

                # Increment our index
                bank_no += 1

            # Update our sequence
            sequence = UltraSync.next_sequence(
                zone.get('sequence', 0)) \
                if zone.get('bank_state') != self.__zvbank[bank] \
                else zone.get('sequence', 0)

            zone.update({
                'priority': priority,
                'status': status,
                'can_bypass': can_bypass,
                'bank_state': self.__zvbank[bank],
                'sequence': sequence,
            })
        return True

    def _zones(self):
        """
        Parses the Zone from our Ultrasync Panel
        """

        if not self.session_id and not self.login():
            return False

        logger.info('Retrieving initial Zone/Sensor information.')

        # Perform our Query
        response = self.__get('/user/zones.htm', is_json=False)
        if not response:
            return False

        #
        # Get our Zone Sequence
        #
        # It looks like this in the zone.htm response:
        #  var zoneSequence = [110,0,2,73,8,38,0,0,0,10,83,36,0,0,0,0,16,0];
        match = re.search(
            r'var zoneSequence\s*=\s*(?P<sequence>[^]]+]);.*', response, re.M)
        if not match:
            # No match and/or bad login
            return False
        # Store our sequence
        self.__zsequence = json.loads(match.group('sequence'))

        #
        # Get our Zone Sequence
        #
        # It looks like this in the zone.htm response:
        #  var zoneStatus = ["0100000000...000000"];
        match = re.search(
            r'var zoneStatus\s*=\s*(?P<states>[^]]+]);.*', response, re.M)
        if not match:
            # No match and/or bad login
            return False
        self.__zbank = json.loads(match.group('states'))

        #
        # Get our Zone Names
        #
        # It looks like this in the zones.htm response:
        #  var zoneNames = ["Front%20door%20","Garage%20Door","..."];
        match = re.search(
            r'var zoneNames\s*=\s*(?P<zone_names>[^]]+]);.*', response, re.M)
        if not match:
            # No match and/or bad login
            return False

        # Store our Zones ('%21' == '!'; these are un-used sensors)
        self.zones = \
            {x: {'name': unquote(y).strip()
                 if unquote(y).strip() else '{}'.format(x + 1), 'bank': x}
             for x, y in enumerate(json.loads(match.group('zone_names')))
             if y != '%21'}

        #
        # Get our Master Status
        #
        # It looks like this in the zone.htm response:
        #  var master = 1; # 1 if master, 0 if not
        match = re.search(
            r'var ismaster\s*=\s*(?P<flag>[01]+).*', response, re.M)
        if not match:
            # No match and/or bad login
            return False
        self.__is_master = True if int(match.group('flag')) else False

        #
        # Get our Installer Status
        #
        # It looks like this in the zone.htm response:
        #  var installer = 1; # 1 if installer, 0 if not
        match = re.search(
            r'var isinstaller\s*=\s*(?P<flag>[01]+).*', response, re.M)
        if not match:
            # No match and/or bad login
            return False
        self.__is_installer = True if int(match.group('flag')) else False

        return self.process_zones()

    def _area_status_update(self, bank=0):
        """
        Performs a status check

        A status response could look like this:
        {
            "time":"5F834156",
            "abank":0,
            "aseq":109,
            "bankstates":"0100000000000000000000000000000000000100\
                          0000000000000000000000000000000000000000",
            "entry":[0,0,0,0],
            "exit":[0,0,0,0],
            "system":[],
            "zwtmp":[]
        }

        """

        logger.info(
            'Updating Area information on bank {}'.format(bank))

        if not self.session_id and not self.login():
            return None

        payload = {
            'sess': self.session_id,

            # Select the Area Bank
            'arsel': bank,
        }

        # Perform our Query
        response = self.__get('/user/status.json', payload=payload)
        if not response:
            return None

        # Update our bank states
        self.areas[bank]['bank_state'] = response['bankstates']

        # Convert Hex time to Local Date Time
        response['time'] = datetime.fromtimestamp(
            int(response['time'], 16)).strftime('%Y-%m-%d %H:%M:%S')

        return response

    def _zone_status_update(self, bank=0):
        """
        Performs a zone status check

        A status response could look like this:
        {
            "time":"5F8DE0C6",
            "zbank":0,
            "zseq":134,
            "bankstates":"000000000000000000000000",
            "system":[]
        }
        """

        logger.info(
            'Updating Zone/Sensor information on bank {}'.format(bank))

        if not self.session_id and not self.login():
            return None

        payload = {
            'sess': self.session_id,

            # Select the Zone Bank
            'state': bank,
        }

        # Perform our Query
        response = self.__get('/user/zstate.json', payload=payload)
        if not response:
            return None

        # Update our bank states
        self.__zbank[bank] = response['bankstates']

        # Convert Hex time to Local Date Time
        response['time'] = datetime.fromtimestamp(
            int(response['time'], 16)).strftime('%Y-%m-%d %H:%M:%S')

        return response

    def _sequence(self):
        """
        Returns the sequences for both the zones, entries, and areas

        A sequence response could look like this:
        {
            "time":"5F83415C",
            "area":[50,0,0,0,0,0,0,0,0,0,0,0],
            "zone":[82,1,3,242,0,0,0,0,0,0,0,0,0,0,0,0,0,0],
            "chime":[3,0,0,0],
            "config":12345,
            "zwdevice":67890
        }

        A sequence is a way of polling the panel to see if there is anything
        new to report. If there are no changes, then the sequence values
        remain unchanged.

        If a sequence value is changed, the index of the 'area' is the bank
        that was updated.... so if index 0 was updated, then Area 1 changed.
        It is up to the user to call for an _area_status_update() with the
        respected index that needs updating.

        The same goes for if a 'zone' bank changes; a call to
        _zone_status_update() is required to be called referencing the
        respected bank that changed.

        """
        if not self.session_id and not self.login():
            return None

        # Perform our Query
        response = self.__get('/user/seq.json')
        if not response:
            return None

        # Convert Hex time to Local Date Time
        response['time'] = datetime.fromtimestamp(
            int(response['time'], 16)).strftime('%Y-%m-%d %H:%M:%S')

        perform_area_update = False
        perform_zone_update = False

        # Process Zones/Sensors first
        for bank, sequence in enumerate(self.__zsequence):
            if sequence != response['zone'][bank]:
                logger.debug('Zone {} sequence changed'.format(bank + 1))
                # We need to update our status here
                self.__zbank[bank] = response['zone'][bank]
                self._zone_status_update(bank=bank)
                perform_zone_update = True

        # Process Area now
        for bank, area in self.areas.items():
            if area['sequence'] != response['area'][bank]:
                logger.debug('Area {} sequence changed'.format(bank + 1))
                # We need to update our status here
                area['sequence'] = response['area'][bank]
                self._area_status_update(bank=bank)
                perform_area_update = True

        if perform_zone_update:
            # Update our zone sequence
            self.__zsequence = response['zone']

            # Process all of our triggered zones/sensors
            self.process_zones()

        if perform_area_update:
            # Process all of our triggered areas
            self.process_areas()

        return response

    def __get(self, path, payload=None, is_json=True, method='POST',
              auth_on_fail=True):
        """
        HTTP POST wrapper for interfacing with our panel
        """

        headers = {
            'Referer': 'http://{}/login.htm'.format(self.host),
            'User-Agent': self.user_agent
        }

        if payload is None:
            # Our default session
            payload = {
                'sess': self.session_id,
            }

        # Prepare our URL
        url = 'http://{}{}'.format(self.host, path)

        logger.trace('{} Request to {}'.format(method, url))

        request = None

        try:
            if method == 'POST':
                # Make our POST request
                request = self.session.post(
                    url, data=payload, headers=headers, timeout=self.timeout,
                    allow_redirects=False)

            else:
                # Make our request
                request = self.session.get(
                    url, data=payload, headers=headers, timeout=self.timeout,
                    allow_redirects=False)

            logger.trace('URL: {}, status_code: {}'.format(
                url, request.status_code))
            logger.trace('URL: {}, response:\n{}'.format(
                url, request.content.decode(self.panel_encoding)))

        except requests.exceptions.ReadTimeout:
            logger.error('Connection read timeout')

        except requests.exceptions.ConnectTimeout:
            logger.error('Connection timeout')

        except requests.exceptions.ConnectionError:
            logger.error('Connection error')

        if auth_on_fail and (
                request is None or
                request.status_code == requests.codes.found):
            # A redirect implies that we were logged out
            # we also treat flat out failed connections as a logout
            # and attempt one more time...

            if not self.login():
                # We couldn't get back in; now we're definiely done
                return False

            # Recursively try our query again...

            if 'sess' in payload:
                # Update our session id
                payload['sess'] = self.session_id

            return self.__get(
                path, payload=payload, is_json=is_json, method=method,
                auth_on_fail=False)

        if not request or request.status_code != requests.codes.ok:
            logger.error('Failed to query {}'.format(url))
            return None

        # Our response object
        response = None

        if is_json:
            try:
                response = json.loads(
                    request.content.decode(self.panel_encoding))

            except (AttributeError, TypeError, ValueError):
                # ValueError = request.content is Unparsable
                # TypeError = request.content is None
                # AttributeError = r is None
                logger.error(
                    'Failed to receive JSON response from {}'
                    .format(url))
        else:
            response = request.content.decode(self.panel_encoding)

        # Return our results
        return response

    @staticmethod
    def next_sequence(last):
        """
        Returns a rotating sequence number (used for tracking changes that
        occur with zone/sensor data)
        """
        if last < 256:
            return last + 1
        return 0



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019 Chris Caron <lead2gold@gmail.com>
# All rights reserved.
#
# This code is licensed under the MIT License.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files(the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and / or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions :
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import mock
import requests
from os.path import join
from os.path import dirname

from ultrasync import UltraSync

# Disable logging for a cleaner testing output
import logging
logging.disable(logging.CRITICAL)

# Reference Directory
TEST_VAR_DIR = join(dirname(__file__), 'var')


@mock.patch('requests.Session.post')
def test_login(mock_post):
    """
    test login

    """

    # A response object
    robj = mock.Mock()

    # Simulate a valid login return
    with open(join(TEST_VAR_DIR, 'login.cgi-okay'), 'rb') as f:
        robj.content = f.read()
    robj.status_code = requests.codes.ok

    # Assign our response object to our mocked instance of requests
    mock_post.return_value = robj

    uobj = UltraSync()
    assert uobj.login()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# All rights reserved.
#
# This code is licensed under the MIT License.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files(the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and / or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions :
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN

import os
import re

from .logger import logger


class UltraSyncConfig(object):
    """
    A very simple class that takes a file and extracts what it needs from
    it in order to work the UltraSync() object
    """

    # Default values
    _host = 'zerowire'
    _user = 'User 1'
    _pin = '1234'

    _user_agent = 'Mozilla/5.0 (X11; Fedora; Linux x86_64; rv:69.0) ' \
                  'Gecko/20100101 Firefox/69.0'

    def __init__(self, pin=None, user=None, host=None, path=None,
                 user_agent=None, *args, **kwargs):
        """
        Initializes the configuration object
        """

        # Assign Defaults
        self._pin = pin if pin else UltraSyncConfig._pin
        self._user = user if user else UltraSyncConfig._user
        self._host = host if host else UltraSyncConfig._host
        self._user_agent = \
            user_agent if user_agent else UltraSyncConfig._user_agent

        if path and not self.load(path):
            raise AttributeError('Invalid path specified: {}'.format(path))

    def load(self, path=None):
        """
        Loads the configuration specified by the path
        """

        # Define what a valid line should look like
        valid_line_re = re.compile(
            r'^\s*(?P<key>[^:#;\s]+)\s*:\s*(?P<value>.+)$', re.I)

        try:
            # Open/Parse File
            with open(path, 'r') as reader:
                line = '\n'
                while line:
                    line = reader.readline()
                    match = valid_line_re.match(line)
                    if not match:
                        # keep reading
                        continue
                    key = '_{}'.format(match.group('key'))
                    if hasattr(self, key):
                        setattr(self, key, match.group('value'))
                        logger.trace('Config: {} loaded as {}'.format(
                            key[1:], match.group('value')))

        except (IOError, OSError, TypeError):
            # Could not load configuration
            return False

        return True

    @property
    def host(self):
        """
        Returns environment variable ULTRASYNC_HOST if defined otherwise
        it falls back to the parsed content.
        """
        return os.environ.get('ULTRASYNC_HOST', self._host)

    @property
    def user(self):
        """
        Returns environment variable ULTRASYNC_USER if defined otherwise
        it falls back to the parsed content.
        """
        return os.environ.get('ULTRASYNC_USER', self._user)

    @property
    def pin(self):
        """
        Returns environment variable ULTRASYNC_PIN if defined otherwise
        it falls back to the parsed content.
        """
        return os.environ.get('ULTRASYNC_PIN', self._pin)

    @property
    def user_agent(self):
        """
        Returns environment variable ULTRASYNC_PIN if defined otherwise
        it falls back to the parsed content.
        """
        return os.environ.get('ULTRASYNC_USER_AGENT', self._user_agent)



# -*- coding: utf-8 -*-
#
# All rights reserved.
#
# This code is licensed under the MIT License.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files(the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and / or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions :
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

# defaultNames Array() - v_ZW_03.02-C/en_us.js
DEFAULT_NAMES = (
    '',
    '',
    'Sensor',
    'Area',
    'Channel',
    '',
    'Schedule',
    'Action',
    'Arm-Disarm',
    '',
    'Permission',
    'Area Group',
    'Menu',
    'Holiday',
    'Sensor Type',
    'Sensor Options',
    'Event List',
    'Channel Group',
    'Action group',
    'Scene',
    '',
    'Camera',
    'UltraSync',
)

# groupingNames Array() - v_ZW_03.02-C/en_us.js
GROUPING_NAMES = (
    'Select Sensor to Configure',
    'Select Keyfob to Configure',
    'Select Area to Configure',
    'Area Timers',
    'Area Options',
    'Area Reporting',
    'System Date and Time',
    'System Time Zone',
    'System Daylight Saving Time',
    'System Timers',
    'System Options',
    'System Reporting',
    'Select Channel to Configure',
    'LAN configuration',
    'WiFi Configuration',
    'Remote Access PINS',
    'Options',
    'Select Scene to Configure',
    'Scene Trigger',
    'Scene Result',
    'Select Schedule to Configure',
    'Time and Days 1',
    'Time and Days 2',
    'Time and Days 3',
    'Time and Days 4',
    'Select Holiday List to Configure',
    'Start Date',
    'End Date',
    'Connection Status',
    'Cellular Radio Details',
    'WiFi Details',
    'Device Details',
    'Camera Configuration',
    'Camera Network Configuration',
)

# firstArray Array() - v_ZW_03.02-C/en_us.js
FIRST = (
    'Disabled',
    'Always On',
    'Backup Channel Only',
)

# setNames Array() - v_ZW_03.02-C/en_us.js
NAMES = (
    'Sensors',
    'Keyfobs',
    '',
    'Areas',
    'System',
    'Reporting and Notifications',
    'Network',
    'Automations (Scenes)',
    'Schedules',
    'Holidays',
    'Z-Wave Room Names',
    'Z-Wave Add/Remove',
    'Z-Wave Device Association',
    'Z-Wave Maintenance',
    'WiFi Setup',
    'Cameras',
    'Connection Status',
    'Details',
    'Walk Test',
    'Lock PIN Share',
)

SCENE_STATES = (
    'Disabled',
    'Sensor Bypass',
    'Arm Away',
    'Disarm',
    'Arm Stay',
    'Restart AutoArm Timer',
    'Arm Away - No Auto Stay',
    'Chime On',
    'Chime Off',
    'Activate Scene',
    'Trigger Camera Video Clip',
    'Send Area Notification',
    'Send Sensor Notification',
    'Send Notification',
    'Start Siren',
)

SCENE_ACTIVATION = (
    'Disable',
    'Sensor Open',
    'Sensor Not Open',
    'Sensor Alarm',
    'Area Armed Away',
    'Area Armed + Bypass',
    'Area Armed Stay',
    'Area Not Armed Away',
    'Entry Delay',
    'Exit Delay 1',
    'Exit Delay 2',
    'Area Sensor Bypass',
    'Area Tamper',
    'Area Not Ready to Arm',
    'Area Sensor Low Battery',
    'Area Sensor Supervision Fault',
    'Area Alarm',
    'Area Burg Alarm',
    'Area Fire Alarm',
    'Area Panic Alarm',
    'Area Auxiliary Alarm',
    'Area Siren',
    'Area Fire Siren',
    'User PIN entered',
    'Action Function True',
    'Action Function False',
    'Schedule Activated',
    'Schedule Deactivated',
    'Smoke Power Reset',
    'Turn On By User',
    'Turn Off By User',
    'Geo Radius Entered',
    'Geo Radius Exited',
    '!',
    '!',
    'Key A',
    'Key B',
    'Key C',
    'Siren On',
    'Z-Wave Devices (Beta)',
    'Sunrise',
    'Sunset',
    'Camera Motion Detection',
    'Doorbell Camera Button Press',
    '!',
)

# voicetokens Array() - v_ZW_03.02-C/en_us.js
VOICE_TOKENS = (
    'ZERO',
    'ONE',
    'TWO',
    'THREE',
    'FOUR',
    'FIVE',
    'SIX',
    'SEVEN',
    'EIGHT',
    'NINE',
    'TEN',
    'ELEVEN',
    'TWELVE',
    'THIRTEEN',
    'FOURTEEN',
    'FIFTEEN',
    'SIXTEEN',
    'SEVENTEEN',
    'EIGHTEEN',
    'NINETEEN',
    'TWENTY',
    'THIRTY',
    'FORTY',
    'FIFTY',
    'SIXTY',
    'SEVENTY',
    'EIGHTY',
    'NINETY',
    'HUNDRED',
    'THOUSAND',
    'AIR CONDITIONER',
    'AREA',
    'ATTIC',
    'AUTOMATIC',
    'AUXILLARY',
    'BACK',
    'BASEMENT',
    'BATHROOM',
    'BEDROOM',
    'BOAT',
    'CABINET',
    'CAR PARK',
    'CEILING',
    'CELLAR',
    'CHILDS',
    'ALERT',
    'CLOSET',
    'COMPUTER',
    'COOL',
    'CURTAIN',
    'DATA',
    'DEN',
    'DETECTOR',
    'DINING',
    'DOOR',
    'DOWNSTAIRS',
    'DRIVEWAY',
    'DURESS',
    'EAST',
    'EMERGENCY',
    'ENTRY',
    'FAMILY',
    'FAN',
    'FENCE',
    'FIRE',
    'FORCED ARM',
    'FOYER',
    'FREEZER',
    'FRONT',
    'GAMES',
    'GARAGE',
    'GAS',
    'GATE',
    'GLASS',
    'GLASSBREAK',
    'GROUND',
    'GUEST',
    'GUN',
    'GYM',
    'HALL',
    'HALLWAY',
    'HEAT',
    'HEATING',
    'HOLDUP',
    'HOME',
    'HOME THEATRE',
    'INFRARED',
    'INSIDE',
    'INSTANT',
    'INTERIOR',
    'KEYSWITCH',
    'KEYCHAIN',
    'KITCHEN',
    'LARGE',
    'LAUNDRY',
    'LIFT',
    'LIGHT',
    'LIVING',
    'LOCATION',
    'MASTER',
    'MEDICINE',
    'MEETING',
    'MOTION',
    'NIGHT',
    'NORTH',
    'NURSERY',
    'OFFICE',
    'OUTPUT',
    'OUTSIDE',
    'PANIC',
    'PANTRY',
    'PARTIAL',
    'PERIMETER',
    'POOL',
    'REAR',
    'RECEPTION',
    'REMOTE',
    'ROOF',
    'ROOM',
    'RUMPUS',
    'SAFE',
    'SECURITY',
    'SENSOR',
    'SHED',
    'SHOCK',
    'SHOP',
    'SIDE',
    'SKYLIGHT',
    'SLIDING',
    'SMALL',
    'SMOKE',
    'SOUTH',
    'STAIRS',
    'STORAGE',
    'STUDY',
    'TEMPERATURE',
    'SPARE3',
    'TOILET',
    'TRAINING',
    'TV',
    'UPSTAIRS',
    'USER',
    'UTILITY',
    'VOLT',
    'VERANDA',
    'WALL',
    'WAREHOUSE',
    'WATER',
    'WEST',
    'WINDOW',
    'WINDOWS',
    'WIRELESS',
    'YARD',
    '',
)

# languageStrings Array() - v_ZW_03.02-C/en_us.js
LANGUAGE_LOOKUP = (
    'Configuration Server',
    'Back',
    'Up',
    'Down',
    'Save',
    'All On',
    'All Off',
    'Shortcut',
    'disabled',
    'ON',
    'OFF',
    'Sensor Add/Remove Functions',
    'Learn',
    'Remove',
    'Cancel',
    'Select Menu:',
    'KeyFob',
    'Keypad',
    'Channel',
    'IP Address',
    'Gateway',
    'Subnet',
    'Primary DNS',
    'Secondary DNS',
    'Date',
    'Time',
    'Standard User Type (Fob Number)',
    'Discard Changes?',
    'Select a Room to Edit Names',
    'You must select a Menu before you can scroll',
    'Select a submenu from the list or select back to access the main menu',
    'Data limited to',
    'digits',
    'characters',
    'Data must only contain the following characters',
    'Date must be of the form YYYY-MM-DD.',
    'Month must be from 1 to 12 Use Get Calendar for easy formatting',
    'Day must be from 1 to 31',
    'Data entry must only contain the numbers 0 - 9 and A-F',
    'Data entry must only contain the numbers 0 - 9',
    'Data must be a number from',
    'To',
    'Enter Menu Shortcut',
    'Defaulting requires 2 levels',
    'Improper Time Value',
    'Settings Selector',
    'Reload',
    'Write Access Denied',
    'Nothing displayed can be Saved',
    'Program Success!',
    'Name Saved',
    'Select a device to Edit Names',
    'Or Click A Function Above',
    'Saving Room Name',
    'Room Name Saved',
    'Select a Room to Edit Name',
    'WiFi Mode Not Enabled',
    'Scan For WiFi Networks',
    'Are you sure you want to change the sensor ID and/or type?',
    'Activating Learn Mode',
    'New Device Found',
    'Click Save to Store New Device',
    'Learn Mode Active',
    'Activate Learn Button',
    'Wrong Device Type',
    'Activate Learn Mode Button on another device',
    'Are you sure you want to permanently delete Sensor?',
    'Use FOB Number as Standard User',
    'Please enter your WEP key',
    'No key entered',
    'WEP key is not the correct length',
    'Keys must be either:',
    '(a) 10 or 26 hexadecimal digits or',
    '(b) 5 or 13 ASCII characters',
    'Please enter your passphrase',
    'No passphrase entered',
    'WPA passphrase is not the correct length',
    'Passphrase must be 8-63 characters long',
    'SSID cannot be left blank!',
    'Please choose either adhoc or infrastructure',
    'This Page Is Under Construction',
    'System Name and Clock',
    'Area Name and Entry/Exit Times',
    'Changes Saved',
    'must be 4 to 8 digits',
    'Scan For New Cameras',
    'Unlocked',
    'Locked',
    'Unlocking…',
    'Locking…',
    'Timed Unlock',
    'Timed Unlock',
    'Open',
    'Closed',
    'Opening…',
    'Closing…',
    'Stopped',
    'Safety Delay',
    'Safety Delay',
    'Any User',
    'Voice Name',
    'Are you sure you want to delete camera',
    'High Temp',
    'Low Temp',
    'Select one or more triggers',
    'Enable External Contact',
    'Would you like to Supervise Output 2 as Sensor',
    'Would you like to Use input 1 as Sensor',
    'Would you like to Use input 2 as Sensor',
    'I/O Module',
    'Indoor Siren',
    'Outdoor Siren',
    'Transmitter Type',
)

# addRemStatus Array() - v_ZW_03.02-C/en_us.js
ADD_REMOVE_STATUS = (
    'Learn Ready. Activate Device Learn Sequence or Press Cancel',
    'Learn Ready. Activate Device Learn Sequence or Press Cancel',
    'Device Found',
    'Slave Found',
    'Adding Controller',
    'Protocol Query Done',
    'Done',
    'Failed',
)

# operatingModes Array() - v_ZW_03.02-C/en_us.js
OPERATING_MODES = (
    'OFF',
    'HEAT',
    'COOL',
    'AUTO',
    'E-HEAT',
    'RESUME',
    'FAN ONLY',
    'FURNACE',
    'DRY AIR',
    'MOIST AIR',
    'AUTOCH',
    'ESAVE-H',
    'ESAVE-C',
    'AWAY',
)

# operatingStates Array() - v_ZW_03.02-C/en_us.js
OPERATING_STATES = (
    'IDLE',
    'HEAT',
    'COOL',
    'FAN ONLY',
    'PENDING-HEAT',
    'PENDING-COOL',
    'VENT-ECONOMIZE',
)

# fanmodeNames Array() - v_ZW_03.02-C/en_us.js
FAN_MODES = (
    'AUTO',
    'ON',
    'AUTO-HIGH',
    'ON-HIGH',
    'AUTO-MED',
    'ON-MED',
    'CIRCULATE',
    'HUMIDIFY',
)

# zwaveStrings Array() - v_ZW_03.02-C/en_us.js
ZWAVE = (
    'Unavailable - Failed Device Function in progress',
    'Unavailable - Add mode active',
    'Unavailable - Remove mode active',
    'Unavailable - Resetting Network',
    'Unavailable - Backing Up Network',
    'Unavailable - Restoring Network',
    'Are you sure you want to delete all Devices?',
    'Are you sure you want to Restore a previous backup?',
    'Busy, Try Again Momentarily',
    'Not primary controller',
    'No Call Back Function',
    'Device Not found in failed list',
    'Remove Device failed - already in process',
    'Replace Device failed - already in process',
    'Remove Device Failed',
    'Replace Device Failed',
    'Device Removed',
    'Remove Failed',
    'Device ',
    ' Replaced',
    'Replace Failed',
    'Function timed out or cancelled',
    'Added Device ',
    'Enter Location Name and Device Name then click Save Device Name',
    'Removed Device ',
    'Select a device to Edit Names Or Click A Function Above',
    'No Device Added',
    'No Device Removed',
    'Network Reset ',
    'Select a device to Edit Names Or Click A Function Above',
    'Network Backed Up',
    'Network Restored',
    'Unavailable, Try Again Later',
    'Learn Ready',
    'Activate Device Learn Sequence on new Device ',
    ' Device',
    'Adding - ',
    'Removing - ',
    'Processing Request',
    'Resetting Network',
    'Backing Up Network',
    'Restoring Network',
    ' Associations',
    'Edit the Location and Device Name',
    'Then Click Save Device Name',
    'Or Click A Function Above',
    'Off',
    'On',
    'Turn Off',
    'Turn On',
    'Updating Status',
    'Command Failed',
    'Current Level ',
    'Association Add Done',
    'Association Remove Done',
    'Association Query Done',
    'FAN MODES',
    'Temperature',
    'Press Select for Set Points',
    'Fan Mode',
    'You must press Select to choose a set point',
    'You must select a failed Device other than 1',
    'Unable To Retrieve Thermostat Information',
    'SET POINT',
    'SELECT',
    'DOWN',
    'UP',
    'OPERATING MODES',
    'UNLOCK',
    'LOCK',
    'Room Selector',
    'All Lights',
    'All HVAC',
    'All Doors',
    'Z-Wave Device Selector',
    'Z-Wave Room Editor',
    'Device Room Location',
    'Device Name',
    'Add',
    'Remove',
    'Cancel',
    'Tick For High Power Add Option',
    'Device Add/Remove Functions',
    'Association Group',
    'Query',
    'Replace',
    'Remove',
    'Network Maintenance Functions',
    'Backup',
    'Restore',
    'Reset',
    'Adding devices to association',
    'Removing devices from association',
    'Querying for association devices',
    'Querying for failed devices',
    'Select Association Group',
    'Then Select Device',
    'Then Click Function Button',
    'Association Functions',
    'Failed Device Functions',
    'Failed Device Selector',
    'Association Selector',
    'There are no Failed Devices',
    'Include',
    'Open',
    'Close',
    'Mode',
    'Operating State',
    'Shift',
    'Exclude',
    'Replacing - ',
    'Room',
    'Unavailable - Rediscovering Network',
    'Rediscovery Complete',
    'Rediscovering',
    'Node Discovery Functions',
    'Rediscover',
    'Restart',
    'Select Door Lock',
    'All Users',
    'Select User(s)',
    'Send PIN(s) to Lock',
    'Remove PIN(s) from Lock',
    'Message Center',
    'Lock PIN Share Instructions',
    'Sending',
    'Removed',
    'Sent',
    'Building User List- Please Wait',
    '1. Select Door Lock.',
    '2. Select User(s).',
    '3. Press Send or Remove Function Button.',
    '4. Repeat Steps 1-3 as necessary.',
    '',
    'All PINS Updated',
    'Edit Room Name',
    'Are you sure you want to add this control to another Z-Wave network?',
    'Alarm is Primary Controller',
    'Alarm is Secondary Controller',
    'Error Occurred! Remove New Device and try again.',
)

# setPointNames Array() - v_ZW_03.02-C/en_us.js
POINT_NAMES = (
    'Not Used',
    'Heat',
    'Cool',
    'Furnace',
    'Dry Air',
    'Moist Air',
    'Auto Changeover',
    'E-Save Heat',
    'E-Save Cool',
    'Away Heat',
)

# areaStates Array() - v_ZW_03.02-C/en_us.js
AREA_STATES = (
    'Armed Away',
    'Armed Stay',
    'Ready',
    'Fire Alarm',
    'Burg Alarm',
    'Panic Alarm',
    'Auxiliary Alarm',
    'Exit Delay',
    'Exit Delay 2',
    'Entry Delay',
    'Sensor Bypass',
    'Sensor Trouble',
    'Sensor Tamper',
    'Sensor Low Battery',
    'Sensor Supervision',
    '',
)

# zoneStates Array() - v_ZW_03.02-C/en_us.js
ZONE_STATES = (
    'Not Ready',
    'Tamper',
    'Trouble',
    '',
    'Inhibited',
    'Alarm',
    'Low Battery',
    'Supervision Fault',
    'Test Fail',
    '',
    'Entry Delay',
    '',
    'Test Active',
    'Activity Fail',
    'Antimask',
)

# statusStrings Array() - v_ZW_03.02-C/en_us.js
STATUS_STATES = (
    'Updating System Data',
    'No System Faults',
    'No Areas Configured For Your Access',
    'All Areas',
    'Area',
    'Bypass',
    'Chime',
    'Ready',
    'Sensor',
    'No Sensors Configured For Your Access',
    'Away',
    'Stay',
    'Off',
    'Notify',
)

# pinStrings Array() - v_ZW_03.02-C/en_us.js
PIN = (
    'Add',
    'Edit',
    'Delete',
    'Save',
    'Entry must only contain the numbers 0 - 9',
    'You must enter a user Number between 1 and 999',
    'PIN digits must be between 0 and 9',
    'PIN must be from 4 to 8 digits!',
    'User Number',
    'Midnight',
    'Noon',
    'AM',
    'PM',
    'Schedule',
    'Select User',
    'First Name',
    'Last Name',
    'PIN',
    'User Type',
    'Standard',
    'Master',
    'Installer',
    'Master Installer',
    'Arm Only',
    'Duress',
    'Custom',
    'Start',
    'End',
    'Profile',
    'Pin Must be 4-8 digits from 0-9',
    'Configure Users',
    'Change PIN',
    'Sort by Name',
    'Language',
    'Minimum PIN length is',
    'Maximum PIN length is 8',
    'Display Area List',
    'Area Type Override',
    '',
    '',
)

# masterStrings Array() - v_ZW_03.02-C/en_us.js
MASTER = (
    'Connection Was lost before a response was received',
    'Try Switching to the new Network',
    'Menu',
    'Logout',
    'Arm/Disarm',
    'Sensors',
    'Cameras',
    'Rooms',
    'History',
    'Users',
    'Change PIN',
    'Settings',
    'Advanced',
    'Data must not contain the following characters',
    'No Cameras Configured.',
    'Contact your security and home automation dealer to add live video and '
    'recorded clips.',
    'No Camera Access On Local Area Network.',
    'Enter Your Name',
    'Enter Your Password',
    'Sign In',
    'Not Ready',
    'Not Ready - Force Armable',
    'Network Successfully Selected!',
    'Change your computer to the same network before Attempting to reconnect.',
    '',
)

# historyStrings Array() - v_ZW_03.02-C/en_us.js
HISTORY = (
    '|<',
    '<',
    '>',
    '>|',
    'Search',
    'Please select a date',
    'No Events Match Search Criteria',
    'Event History',
    'Select Date',
    'All Events',
    'Alarm Events',
    'Video Events',
    'Event Filter',
    'Press to Play',
)

# walktestStrings Array() - v_ZW_03.02-C/en_us.js
WALK_TEST = (
    'Walk Test Instuctions',
    'Start Walk Test',
    'End Walk Test',
    'Result',
    'Pass',
    'Fail',
    'To Test:',
    '1. Open (Trigger) Sensor',
    '3. Close (Restore) Sensor',
    '5. Check Result',
    'Current Noise Floor',
    'Peak Noise Floor',
    'Best Noise Floor',
    'Open (Trigger) Packets',
    'Open (Trigger) Signal Strength',
    'Close (Restore) Packets',
    'Close (Restore) Signal Strength',
    'Retrieving Data…',
    '6. Repeat Steps 1-5 for all sensors',
    'Wait 8 Seconds',
    'Excellent',
    'Good',
    'Fair ',
    'Poor',
    'None',
    'Siren Chirp ',
    'Downlink Signal Strength',
    'Uplink Signal Strength',
    'Module Noise Floor',
)



# -*- coding: utf-8 -*-

# Copyright (C) 2020 Chris Caron <lead2gold@gmail.com>
# All rights reserved.
#
# This code is licensed under the MIT License.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files(the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and / or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions :
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

class PanelFunction(object):
    """
    Function Commands (fnum entries)
    """

    AREA_DISARM = 0
    AREA_STAY = 1
    AREA_CHIME_TOGGLE = 10
    AREA_AWAY = 15


class AreaBank(object):
    """
    Defines the Bank Identifiers for Area Queries

    By identifying the bank and breaking it down, it makes the code
    MUCH easier to read for others.  It also makes the code easier to
    debug down the road.

    This list has been purely generated through reverse engineering the
    information available to me using the existing Ultrasync Alarm Panel
    interface.

    """

    UNKWN_00 = 0
    UNKWN_01 = 1
    PARTIAL = 2
    ARMED = 3
    EXIT_MODE01 = 4
    EXIT_MODE02 = 5
    UNKWN_06 = 6
    UNKWN_07 = 7
    UNKWN_08 = 8
    UNKWN_09 = 9
    UNKWN_10 = 10
    UNKWN_11 = 11
    UNKWN_12 = 12
    UNKWN_13 = 13
    UNKWN_14 = 14
    UNKWN_15 = 15
    UNKWN_16 = 16
    UNKWN_17 = 17
    CHIME = 18
    UNKWN_19 = 19
    UNKWN_20 = 20
    UNKWN_21 = 21
    UNKWN_22 = 22
    UNKWN_23 = 23
    UNKWN_24 = 24
    UNKWN_25 = 25
    NIGHT = 26
    UNKWN_27 = 27
    UNKWN_28 = 28
    UNKWN_29 = 29
    UNKWN_30 = 30
    UNKWN_31 = 31
    UNKWN_32 = 32
    UNKWN_33 = 33
    UNKWN_34 = 34
    UNKWN_35 = 35
    UNKWN_36 = 36
    UNKWN_37 = 37
    UNKWN_38 = 38
    UNKWN_39 = 39

class AreaStatus(object):
    """
    Defines the panel display area status
    """

    # All sensor are active; occupants are not present.
    ARMED_AWAY = 'Armed Away'

    # Alarm state when user is present in the home; only perimeter sensors
    # are activated.
    ARMED_STAY = 'Armed Stay'

    READY = 'Ready'

    NOT_READY = 'Not Ready'

    NOT_READY_FORCEABLE = 'Not Ready*'

    DISARMED = 'Disarm'

    EXIT_DELAY_1 = 'Exit Delay 1'

    EXIT_DELAY_2 = 'Exit Delay 2'

    ENTRY_DELAY = 'Entry Delay'


AREA_STATES = (
    # These are intentially set and the order is very important
    # entries missing in this array but defined above is an intentional thing
    AreaStatus.ARMED_AWAY,
    AreaStatus.ARMED_STAY,
    AreaStatus.READY,
    'Fire Alarm',
    'Burg Alarm',
    'Panic Alarm',
    'Auxiliary Alarm',
    AreaStatus.EXIT_DELAY_1,
    AreaStatus.EXIT_DELAY_2,
    AreaStatus.ENTRY_DELAY,
    'Sensor Bypass',
    'Sensor Trouble',
    'Sensor Tamper',
    'Sensor Low Battery',
    'Sensor Supervision',
    '',
)


class ZoneBank(object):
    """
    Defines the Bank Identifiers for Zone/Sensor Queries

    By identifying the bank and breaking it down, it makes the code
    MUCH easier to read for others.  It also makes the code easier to
    debug down the road.

    This list has been purely generated through reverse engineering the
    information available to me using the existing Ultrasync Alarm Panel
    interface.

    """

    UNKWN_00 = 0
    UNKWN_01 = 1
    UNKWN_02 = 2
    UNKWN_03 = 3
    UNKWN_04 = 4
    UNKWN_05 = 5
    UNKWN_06 = 6
    UNKWN_07 = 7
    UNKWN_08 = 8
    UNKWN_09 = 9
    UNKWN_10 = 10
    UNKWN_11 = 11
    UNKWN_12 = 12
    UNKWN_13 = 13
    UNKWN_14 = 14
    UNKWN_15 = 15
    UNKWN_16 = 16
    BYPASS_DISABLED = 17


class ZoneStatus(object):
    """
    Defines the panel display zone/sensor status
    """

    READY = 'Ready'
    NOT_READY = 'Not Ready'

ZONE_STATES = (
    ZoneStatus.NOT_READY,
    'Tamper',
    'Trouble',
    '',
    'Inhibited',
    'Alarm',
    'Low Battery',
    'Supervision Fault',
    'Test Fail',
    '',
    'Entry Delay',
    '',
    'Test Active',
    'Activity Fail',
    'Antimask',
)


class AlarmScene(object):
    """
    Defines the different alarm panel states and/or macros
    """

    # All sensor are active; occupants are not present.
    AWAY = 'away'

    # Alarm state when user is present in the home; only perimeter sensors
    # are activated.
    STAY = 'stay'

    # Alarm system is disarmed
    DISARMED = 'disarm'



# A list of all valid alarm states used for validating
ALARM_SCENES = (
    AlarmScene.AWAY,
    AlarmScene.STAY,
    AlarmScene.DISARMED,
)
